/**************************************************************************************

  Interfacing PIC18F46K22 microcontroller with SSD1306 OLED (128x64 Pixel).
  C Code for mikroC PRO for PIC compiler.
  Internal oscillator used @ 16MHz
  Configuration words: CONFIG1H = 0x0028
                       CONFIG2L = 0x0018
                       CONFIG2H = 0x003C
                       CONFIG3H = 0x0037
                       CONFIG4L = 0x0081
                       CONFIG5L = 0x000F
                       CONFIG5H = 0x00C0
                       CONFIG6L = 0x000F
                       CONFIG6H = 0x00E0
                       CONFIG7L = 0x000F
                       CONFIG7H = 0x0040
  This is a free software with NO WARRANTY.
  https://simple-circuit.com/

***************************************************************************************/

// SSD1306 OLED reset pin definition (if available)
#define SSD1306_RST       RD4_bit
#define SSD1306_RST_DIR   TRISD4_bit

#include <SSD1306OLED.c>  // include SSD1306 OLED display driver source code

char txt[11];

void testdrawcircle(void) {
  unsigned short i;
  for (i = 0; i < SSD1306_LCDHEIGHT; i += 2) {
    SSD1306_DrawCircle(SSD1306_LCDWIDTH/2, SSD1306_LCDHEIGHT/2, i);
    SSD1306_Display();
    delay_ms(1);
  }
}

void testfillrect(void) {
  unsigned short i;
  SSD1306_Color = true;
  for (i = 0; i < SSD1306_LCDHEIGHT/2; i += 3) {
    // alternate SSD1306_Colors
    SSD1306_FillRect(i, i, SSD1306_LCDWIDTH - i*2, SSD1306_LCDHEIGHT - i*2);
    SSD1306_Display();
    delay_ms(1);
    if(SSD1306_Color) SSD1306_Color = false;
    else              SSD1306_Color = true;
  }
  SSD1306_Color = true;
}

void testdrawtriangle(void) {
  unsigned short i;
  for (i = 0; i < SSD1306_LCDHEIGHT/2; i += 5) {
    SSD1306_DrawTriangle(SSD1306_LCDWIDTH/2,     SSD1306_LCDHEIGHT/2 - i,
                         SSD1306_LCDWIDTH/2 - i, SSD1306_LCDHEIGHT/2 + i,
                         SSD1306_LCDWIDTH/2 + i, SSD1306_LCDHEIGHT/2 + i);
    SSD1306_Display();
    delay_ms(1);
  }
}

void testfilltriangle(void) {
  int i;
  SSD1306_Color = true;
  for (i = SSD1306_LCDHEIGHT/2; i > 0; i -= 5) {
    SSD1306_FillTriangle(SSD1306_LCDWIDTH/2,     SSD1306_LCDHEIGHT/2 - i,
                         SSD1306_LCDWIDTH/2 - i, SSD1306_LCDHEIGHT/2 + i,
                         SSD1306_LCDWIDTH/2 + i, SSD1306_LCDHEIGHT/2 + i);
    if(SSD1306_Color)  SSD1306_Color = false;
    else               SSD1306_Color = true;
    SSD1306_Display();
    delay_ms(1);
  }
  SSD1306_Color = true;
}

void testdrawroundrect(void) {
  unsigned short i;
  for (i = 0; i < SSD1306_LCDHEIGHT/2 - 2; i += 2) {
    SSD1306_DrawRoundRect(i, i, SSD1306_LCDWIDTH - 2*i, SSD1306_LCDHEIGHT - 2*i, SSD1306_LCDHEIGHT/4 - i/2);
    SSD1306_Display();
    delay_ms(1);
  }
}

void testfillroundrect(void) {
  unsigned short i;
  SSD1306_Color = true;
  for (i = 0; i < SSD1306_LCDHEIGHT/2 - 2; i += 2) {
    SSD1306_FillRoundRect(i, i, SSD1306_LCDWIDTH - 2*i, SSD1306_LCDHEIGHT - 2*i, SSD1306_LCDHEIGHT/4 - i/2);
    if(SSD1306_Color) SSD1306_Color = false;
    else              SSD1306_Color = true;
    SSD1306_Display();
    delay_ms(1);
  }
  SSD1306_Color = true;
}

void testdrawrect(void) {
  unsigned short i;
  for (i = 0; i < SSD1306_LCDHEIGHT/2; i += 2) {
    SSD1306_DrawRect(i, i, SSD1306_LCDWIDTH - 2*i, SSD1306_LCDHEIGHT - 2*i);
    SSD1306_Display();
    delay_ms(1);
  }
}

void testdrawline() {
  int i;
  for (i = 0; i < SSD1306_LCDWIDTH; i += 4) {
    SSD1306_DrawLine(0, 0, i, SSD1306_LCDHEIGHT - 1);
    SSD1306_Display();
    delay_ms(1);
  }
  for (i = 0; i < SSD1306_LCDHEIGHT; i += 4) {
    SSD1306_DrawLine(0, 0, SSD1306_LCDWIDTH - 1, i);
    SSD1306_Display();
    delay_ms(1);
  }
  delay_ms(250);

  SSD1306_ClearDisplay();
  for (i = 0; i < SSD1306_LCDWIDTH; i += 4) {
    SSD1306_DrawLine(0, SSD1306_LCDHEIGHT - 1, i, 0);
    SSD1306_Display();
    delay_ms(1);
  }
  for (i = SSD1306_LCDHEIGHT - 1; i >= 0; i -= 4) {
    SSD1306_DrawLine(0, SSD1306_LCDHEIGHT - 1, SSD1306_LCDWIDTH - 1, i);
    SSD1306_Display();
    delay_ms(1);
  }
  delay_ms(250);

  SSD1306_ClearDisplay();
  for (i = SSD1306_LCDWIDTH - 1; i >= 0; i -= 4) {
    SSD1306_DrawLine(SSD1306_LCDWIDTH - 1, SSD1306_LCDHEIGHT - 1, i, 0);
    SSD1306_Display();
    delay_ms(1);
  }
  for (i = SSD1306_LCDHEIGHT - 1; i >= 0; i -= 4) {
    SSD1306_DrawLine(SSD1306_LCDWIDTH - 1, SSD1306_LCDHEIGHT - 1, 0, i);
    SSD1306_Display();
    delay_ms(1);
  }
  delay_ms(250);

  SSD1306_ClearDisplay();
  for (i = 0; i < SSD1306_LCDHEIGHT; i += 4) {
    SSD1306_DrawLine(SSD1306_LCDWIDTH - 1, 0, 0, i);
    SSD1306_Display();
    delay_ms(1);
  }
  for (i = 0; i < SSD1306_LCDWIDTH; i += 4) {
    SSD1306_DrawLine(SSD1306_LCDWIDTH - 1, 0, i, SSD1306_LCDHEIGHT - 1);
    SSD1306_Display();
    delay_ms(1);
  }
  delay_ms(250);
}

void testscrolltext(void) {
  SSD1306_ClearDisplay();
  SSD1306_GotoXY(50, 8);
  SSD1306_TextSize(2);
  SSD1306_Print("scroll");
  SSD1306_Display();
  delay_ms(1);

  SSD1306_StartScrollRight(0x00, 0x0F);
  delay_ms(2000);
  SSD1306_StopScroll();
  delay_ms(1000);
  SSD1306_StartScrollLeft(0x00, 0x0F);
  delay_ms(2000);
  SSD1306_StopScroll();
  delay_ms(1000);
  SSD1306_StartScrollDiagRight(0x00, 0x07);
  delay_ms(2000);
  SSD1306_StartScrollDiagLeft(0x00, 0x07);
  delay_ms(2000);
  SSD1306_StopScroll();
}

// main function
void main()
{
  OSCCON = 0x70;      // set internal oscillator to 16MHz
  ANSELC = 0;         // configure all PORTC pins as digital
  ANSELD = 0;         // configure all PORTD pins as digital
  
  delay_ms(1000);     // wait a second
  I2C1_Init(400000);  // initialize I2C communication with clock frequency of 400kHz

  // initialize the SSD1306 OLED with an I2C addr = 0x7A (default address)
  SSD1306_Begin(SSD1306_SWITCHCAPVCC, SSD1306_I2C_ADDRESS);
  // print image buffer
  SSD1306_Display();
  delay_ms(5000);
  SSD1306_ClearDisplay();   // clear the buffer

  SSD1306_Color = true;        // set color to true (ON)
  // draw a single pixel
  SSD1306_DrawPixel(10, 10);
  // Show the display buffer on the hardware.
  // NOTE: You _must_ call SSD1306_Display() after making any drawing commands
  //  to make them visible on the display hardware!
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();      // clear the buffer

  // draw many lines
  testdrawline();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();      // clear the buffer

  // draw rectangles
  testdrawrect();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();      // clear the buffer

  // draw multiple rectangles
  testfillrect();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();     // clear the buffer

  // draw mulitple circles
  testdrawcircle();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();      // clear the buffer

  // draw a white circle, 10 pixel radius
  SSD1306_FillCircle(SSD1306_LCDWIDTH/2, SSD1306_LCDHEIGHT/2, 10);
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();    // clear the buffer

  testdrawroundrect();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();    // clear the buffer

  testfillroundrect();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();   // clear the buffer

  testdrawtriangle();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();   // clear the buffer

  testfilltriangle();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();   // clear the buffer

  // draw scrolling text
  testscrolltext();
  SSD1306_Display();
  delay_ms(2000);
  SSD1306_ClearDisplay();   // clear the buffer

  // text display tests
  SSD1306_Color = 1;
  SSD1306_TextSize(1);
  SSD1306_GotoXY(2, 7);
  SSD1306_Print("hello, world!");

  sprintf(txt, "%.6f", 3.141592);
  SSD1306_GotoXY(2, 16);
  SSD1306_Print(txt);

  SSD1306_TextSize(2);
  SSD1306_GotoXY(2, 26);
  sprintf(txt, "0x%LX", 0xDEADBEEF);
  SSD1306_Print(txt);
  SSD1306_Display();
  delay_ms(2000);

  while(1) {
    // invert the display
    SSD1306_InvertDisplay(true);
    delay_ms(1000);
    SSD1306_InvertDisplay(false);
    delay_ms(1000);
  }

}
// end of code.